#!/bin/bash
set -e
# Define a function to print usage
print_usage() {
    echo "Usage: $0 --path <deb_file_path> [--package <package_name>] [--key <gpg_key>] [--keep-file] [--keep-repo]"
    exit 1
}

DEB_FILE_PATH=""
PACKAGE_NAME=""
PUB_KEY_PATTERN=""
KEEP_DEB_FILE=0
KEEP_LOCAL_REPO=0

# Parse options
while [ $# -gt 0 ]; do
    case "$1" in
        --path)
            shift
            if [ -z "$1" ]; then
                print_usage
            fi
            DEB_FILE_PATH="$1"
            shift
            ;;
        --package)
            shift
            if [ -z "$1" ]; then
                print_usage
            fi
            PACKAGE_NAME="$1"
            shift
            ;;
        --key)
            shift
            if [ -z "$1" ]; then
                print_usage
            fi
            PUB_KEY_PATTERN="$1"
            shift
            ;;
        --keep-file)
            KEEP_DEB_FILE=1
            shift
            ;;
        --keep-repo)
            KEEP_LOCAL_REPO=1
            shift
            ;;
        *)
            print_usage
            ;;
    esac
done

# Check if the user provided a path to a .deb file
if [ -z "$DEB_FILE_PATH" ]; then
    print_usage
fi

# Check if the provided file exists and is a .deb file
if [ ! -f "$DEB_FILE_PATH" ] || [ "${DEB_FILE_PATH: -4}" != ".deb" ]; then
    echo "Invalid .deb file path provided."
    exit 1
fi

safe_apt_install() {
    export DEBIAN_FRONTEND=noninteractive
    echo "*begin $1, $2"
    tmp_log=/tmp/sdkm_apt_install_log.${USER}.txt
    sudo rm -f $tmp_log
    set +e; eval "$1" 2>$tmp_log; CMD_STATUS=${PIPESTATUS[0]}; set -e
    MAX_RETRY=3
    TIMES=1
    while [ ${CMD_STATUS} -ne 0 ]; do
        cat $tmp_log > /dev/stderr
        if [ $TIMES -gt $MAX_RETRY ]; then
            echo "Unhandled error when running $1: `tr '\n' '; ' <$tmp_log`" 1>&2; exit 1
        fi
        if grep -q -e "is locked by another process" -e "E: Could not get lock /var/" -e "Could not open lock" $tmp_log; then
            sudo rm -f $tmp_log
            set +e; eval "$1" 2>$tmp_log; CMD_STATUS=${PIPESTATUS[0]}; set -e
            echo "Waiting $2 seconds for apt to be available"; sleep ${2}
        elif grep -q -e "apt --fix-broken install" $tmp_log; then
            echo "safe apt install failed, need to run apt --fix-broken install"
            TIMES=$[ $TIMES + 1 ]
            sudo rm -f $tmp_log
            set +e; sudo apt-get -f -y --allow-downgrades install 2>$tmp_log; CMD_STATUS=${PIPESTATUS[0]}; set -e
            echo "Waiting 5 seconds for apt"; sleep 5
        else
            echo "safe apt install failed: $1, exit code: $CMD_STATUS, retry number: $TIMES"
            echo "will sleep $[ $TIMES*10 ] seconds before next attempt"
            sleep $[ $TIMES*10 ]
            TIMES=$[ $TIMES + 1 ]
            sudo rm -f $tmp_log
            set +e; eval "$1" 2>$tmp_log; CMD_STATUS=${PIPESTATUS[0]}; set -e
        fi
    done
    if [ $CMD_STATUS -eq 0 ] && [ $TIMES -ne 1 ]; then
        echo "safe apt install succeed. retry number: $TIMES."
    fi
}

install_gpg_key() {
    local repo_package_name=$(dpkg -f $DEB_FILE_PATH Package)
    local keyring_file=$(dpkg -L $repo_package_name | grep 'keyring\.gpg$' | head -n 1 | tr -d "\n")
    if [ -n "$keyring_file" ]; then
        if [ -f "$keyring_file" ]; then
            echo "Installing deb repo keyring file"
            sudo cp "$keyring_file" "/usr/share/keyrings/"
        else
            echo "Warning: deb repo keyring file not found" >&2
        fi
    else
        local pub_key_patten
        if [ -n "$PUB_KEY_PATTERN" ]; then
            pub_key_pattern=$PUB_KEY_PATTERN
        else
            pub_key_pattern="\.pub"
        fi
        local pub_key_file=$(dpkg -L $repo_package_name | grep $pub_key_pattern | head -n 1 | tr -d "\n")
        if [ -n "$pub_key_file" ]; then
            echo "Installing deb repo pub key file"
            safe_apt_install "sudo apt-key add $pub_key_file" 5
        else
            echo "Warning: deb repo pub key file not found" >&2
        fi
    fi
    sleep 1
}

# ensure that packages from a local repository have higher priority
set_apt_preference() {
    echo "Setting deb repo preference"
    sudo mkdir -p /etc/apt/preferences.d/
    sudo rm -f /etc/apt/preferences.d/sdkm
    sudo touch /etc/apt/preferences.d/sdkm
    echo "Package: *" | sudo tee -a /etc/apt/preferences.d/sdkm
    echo "Pin: origin \"\"" | sudo tee -a /etc/apt/preferences.d/sdkm
    echo "Pin-Priority: 999" | sudo tee -a /etc/apt/preferences.d/sdkm
}

unset_apt_preference() {
    echo "Unsetting deb repo preference"
    sudo rm -f /etc/apt/preferences.d/sdkm
}

purge_local_repo() {
    if [ $KEEP_LOCAL_REPO -eq 0 ]; then
        echo "Cleaning up local deb repo..."
        local repo_package_name=$(dpkg -f $DEB_FILE_PATH Package)
        safe_apt_install "sudo apt-get purge -y $repo_package_name" 5
    else
        echo "Keeping local deb repo"
    fi
}

install_deb_file() {
    echo "Installing $DEB_FILE_PATH..."
    safe_apt_install "sudo apt-get -y update" 5
    safe_apt_install "sudo apt-get -y --allow-downgrades install $DEB_FILE_PATH" 5
}

install_deb_file

# it's a deb repo if has additional packages to install
if [ -n "$PACKAGE_NAME" ]; then
    set_apt_preference
    install_gpg_key
    safe_apt_install "sudo apt-get -y update" 5
    echo "Installing $PACKAGE_NAME..."
    safe_apt_install "sudo apt-get -y --allow-downgrades install $PACKAGE_NAME" 5
    purge_local_repo
    unset_apt_preference
fi

if [ $KEEP_DEB_FILE -eq 0 ]; then
    echo "Cleaning up deb file..."
    sudo rm -f $DEB_FILE_PATH
fi

echo "Installation complete"
