# Find the default network interface
$defaultRoute = Get-NetRoute -DestinationPrefix "0.0.0.0/0" | Where-Object {$_.NextHop -ne "192.168.55.1"} | Select-Object -First 1
if ($null -eq $defaultRoute) {
    Write-Host 'Could not find route default, device mode setup failed!'
    exit 1
}

$ethPortName = $defaultRoute.InterfaceAlias
$interfaceIndex = $defaultRoute.InterfaceIndex

Write-Host "Found route default $ethPortName, setting up device mode"

# Setup the network prefix
$setupNetwork = "192.168.55.0/24"

# Enable IP forwarding
Set-ItemProperty -Path "HKLM:\\SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters" -Name "IPEnableRouter" -Value 1 -Type DWord

# Setup NAT
try {
    # Remove existing NAT if it exists
    Remove-NetNat -Name "DeviceModeNAT" -Confirm:$false -ErrorAction SilentlyContinue

    # Create new NAT
    New-NetNat -Name "DeviceModeNAT" -InternalIPInterfaceAddressPrefix $setupNetwork
} catch {
    Write-Host "Could not set up NAT. Make sure you're running as Administrator and WinNAT is available."
    exit 1
}

# Setup Windows Firewall rules to allow forwarding
New-NetFirewallRule -DisplayName "Allow Incoming Traffic from $setupNetwork" -Direction Inbound -RemoteAddress $setupNetwork -Action Allow -ErrorAction SilentlyContinue
New-NetFirewallRule -DisplayName "Allow Outgoing Traffic to $setupNetwork" -Direction Outbound -RemoteAddress $setupNetwork -Action Allow -ErrorAction SilentlyContinue

# Check if Google DNS is working
try {
    $result = Resolve-DnsName -Name "nvidia.com" -Server "8.8.8.8" -ErrorAction Stop
    Write-Host 'Device mode on host successfully set with DNS 8.8.8.8!'
    exit 0
} catch {
    exit 1
    Write-Host 'Default Google DNS is not working, trying to find a valid server in local host settings!'

    # Get DNS servers from the interface
    $dnsList = Get-DnsClientServerAddress -InterfaceIndex $interfaceIndex |
               Where-Object {$_.AddressFamily -eq 2} | # IPv4
               Select-Object -ExpandProperty ServerAddresses

    if ($null -eq $dnsList -or $dnsList.Count -eq 0) {
        Write-Host 'Could not get DNS server settings, device mode setup failed!'
        exit 1
    }

    foreach ($dns in $dnsList) {
        try {
            $result = Resolve-DnsName -Name "nvidia.com" -Server $dns -ErrorAction Stop

            # Get the current network adapter (choose the main one you're using)
            $adapter = Get-NetAdapter | Where-Object {$_.Status -eq "Up" -and $_.InterfaceDescription -eq "Remote NDIS Compatible Device"} | Select-Object
            Write-Host "Using adapter: $($adapter.Name)"

            # Add the 8.8.8.8 IP address to your adapter
            New-NetIPAddress -InterfaceIndex $adapter.ifIndex -IPAddress "8.8.8.8" -PrefixLength 32 -ErrorAction SilentlyContinue

            # Create the port forwarding rule
            netsh interface portproxy add v4tov4 listenaddress=8.8.8.8 listenport=53 connectaddress=$dns connectport=53 protocol=tcp

            # Check and start the IP Helper service
            $service = Get-Service -Name iphlpsvc
            if ($service.Status -ne "Running") {
                Write-Host "Starting IP Helper service..."
                Set-Service -Name iphlpsvc -StartupType Automatic
                Start-Service -Name iphlpsvc
            } else {
                Write-Host "IP Helper service is already running"
            }

            # Create firewall rule to allow incoming TCP traffic on port 53
            New-NetFirewallRule -DisplayName "Allow TCP DNS Redirect" -Direction Inbound -Protocol TCP -LocalPort 53 -Action Allow -ErrorAction SilentlyContinue

            Write-Host "Device mode on host successfully set with DNS $dns!"
            exit 0
        } catch {
            # DNS didn't work, try the next one
        }
    }

    Write-Host 'Could not find a valid DNS server, device mode setup failed!'
    Write-Host 'If needed to use device mode, please change the target default dns to a valid server by following the steps in release note.'
    exit 1
}